@@ -4,7 +4,7 @@ kombu.connection
 
 Broker connection and pools.
 
-:copyright: (c) 2009 - 2010 by Ask Solem.
+:copyright: (c) 2009 - 2011 by Ask Solem.
 :license: BSD, see LICENSE for more details.
 
 """
@@ -244,6 +244,9 @@ class BrokerConnection(object):
                             ("transport", transport_cls),
                             ("connect_timeout", self.connect_timeout)))
 
+    def __hash__(self):
+        return hash("|".join(map(str, self.info().itervalues())))
+
     def as_uri(self):
         fields = self.info()
         port = fields["port"]
@@ -391,7 +394,8 @@ class BrokerConnection(object):
         """
         if self._closed:
             return
-        if not self._connection:
+        if not self._connection or not \
+                self.transport.verify_connection(self._connection):
             self._connection = self._establish_connection()
             self._closed = False
         return self._connection
@@ -548,8 +552,7 @@ class ConnectionPool(Resource):
                 self._resource.put_nowait(conn)
 
     def prepare(self, resource):
-        if not resource._connection:
-            resource.connect()
+        resource.connect()
         return resource
 
 
