@@ -10,12 +10,15 @@ Sending and receiving messages.
 """
 from itertools import count
 
+from kombu import entity
 from kombu.compression import compress
-from kombu.entity import Exchange, Queue
 from kombu.serialization import encode
 from kombu.syn import blocking as _SYN
 from kombu.utils import maybe_list
 
+Exchange = entity.Exchange
+Queue = entity.Queue
+
 
 class Producer(object):
     """Message Producer.
@@ -404,12 +407,5 @@ class Consumer(object):
         else:
             self.receive(decoded, message)
 
-    def __enter__(self):
-        self.consume()
-        return self
-
-    def __exit__(self, *args):
-        self.cancel()
-
     def __repr__(self):
         return "<Consumer: %s>" % (self.queues, )
