@@ -62,8 +62,15 @@ class QoS(object):
     prefetch_count = 0
 
     #: :class:`~collections.OrderedDict` of active messages.
+    #: *NOTE*: Can only be modified by the consuming thread.
     _delivered = None
 
+    #: acks can be done by other threads than the consuming thread.
+    #: Instead of a mutex, which doesn't perform well here, we mark
+    #: the delivery tags as dirty, so subsequent calls to append() can remove
+    #: them.
+    _dirty = set()
+
     def __init__(self, channel, prefetch_count=0):
         self.channel = channel
         self.prefetch_count = prefetch_count or 0
@@ -82,24 +89,38 @@ class QoS(object):
 
         """
         pcount = self.prefetch_count
-        return (not pcount or len(self._delivered) < pcount)
+        return (not pcount or len(self._delivered) - len(self._dirty) < pcount)
 
     def append(self, message, delivery_tag):
         """Append message to transactional state."""
         self._delivered[delivery_tag] = message
+        if self._dirty:
+            self._flush()
+
+    def _flush(self):
+        """Flush dirty (acked/rejected) tags from."""
+        dirty = self._dirty
+        delivered = self._delivered
+        while 1:
+            try:
+                dirty_tag = dirty.pop()
+            except KeyError:
+                break
+            delivered.pop(dirty_tag, None)
 
     def ack(self, delivery_tag):
         """Acknowledge message and remove from transactional state."""
-        self._delivered.pop(delivery_tag, None)
+        self._dirty.add(delivery_tag)
 
     def reject(self, delivery_tag, requeue=False):
         """Remove from transactional state and requeue message."""
-        message = self._delivered.pop(delivery_tag)
         if requeue:
-            self.channel._restore(message)
+            self.channel._restore(self._delivered[delivery_tag])
+        self._dirty.add(delivery_tag)
 
     def restore_unacked(self):
         """Restore all unacknowledged messages."""
+        self._flush()
         delivered = self._delivered
         errors = []
 
@@ -122,6 +143,8 @@ class QoS(object):
         Will only be done once for each instance.
 
         """
+        self._on_collect.cancel()
+        self._flush()
         state = self._delivered
 
         if not self.channel.do_restore or getattr(state, "restored"):
@@ -466,7 +489,8 @@ class Channel(AbstractChannel):
         self.closed = True
         for consumer in list(self._consumers):
             self.basic_cancel(consumer)
-        self.qos.restore_unacked_once()
+        if self._qos:
+            self._qos.restore_unacked_once()
         self.connection.close_channel(self)
         if self._cycle is not None:
             self._cycle.close()
